package org.library.app;

import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.Parent;
import javafx.scene.layout.StackPane;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import javafx.stage.Modality;

import java.net.URL;
import java.util.List;
import java.time.LocalDate;

import org.javalite.activejdbc.Base;
import org.javalite.activejdbc.DBException;

import java.io.IOException;
import java.io.File;


public class AppManager {
  private Scene scene;
  private Stage stage;
  private MainController mainController;

  public void initialize() {
  }

  public AppManager(Scene scene, Stage stage) {
    this.scene = scene;
    this.stage = stage;
  }

  public void showLoginView() {
    this.stage.setTitle("Библиотека | Аутентификация");

    URL view_login_path = AppManager.class.getResource("/views/login.fxml");
    FXMLLoader loader = new FXMLLoader(view_login_path);
    Parent root = null;
    try {
      root = loader.load();
    }
    catch (IOException err) {
      err.printStackTrace();
    }

    LoginController controller = loader.<LoginController>getController();
    controller.initController(this);

    this.scene.setRoot(root);
  }

  public void showMainView() {
    this.stage.setTitle("Библиотека");

    URL view_main_path = AppManager.class.getResource("/views/main.fxml");
    FXMLLoader loader = new FXMLLoader(view_main_path);
    Parent root = null;
    try {
      root = loader.load();
    }
    catch (IOException err) {
      err.printStackTrace();
    }

    MainController mainController = loader.<MainController>getController();
    mainController.initController(this);
    this.mainController = mainController;

    this.scene.setRoot(root);
  }

  public void showFormBookAddView() {
    URL view_form_book_add_path = AppManager.class.getResource("/views/modalAddBook.fxml");
    FXMLLoader loader = new FXMLLoader(view_form_book_add_path);
    Parent root = null;
    try {
      root = loader.load();
    }
    catch (IOException err) {
      err.printStackTrace();
    }

    Scene modalScene = new Scene(new StackPane());
    Stage modalStage = new Stage(StageStyle.TRANSPARENT);
    modalStage.initModality(Modality.APPLICATION_MODAL);
    modalScene.setRoot(root);
    modalStage.setScene(modalScene);

    FormBookAddController controller = loader.<FormBookAddController>getController();
    controller.initController(this, modalStage);

    modalStage.show();
  }

  public void showFormBookEditView(Book book) {
    URL view_form_book_edit_path = AppManager.class.getResource("/views/modalEditBook.fxml");
    FXMLLoader loader = new FXMLLoader(view_form_book_edit_path);
    Parent root = null;
    try {
      root = loader.load();
    }
    catch (IOException err) {
      err.printStackTrace();
    }

    Scene modalScene = new Scene(new StackPane());
    Stage modalStage = new Stage(StageStyle.TRANSPARENT);
    modalStage.initModality(Modality.APPLICATION_MODAL);
    modalScene.setRoot(root);
    modalStage.setScene(modalScene);

    FormBookEditController controller = loader.<FormBookEditController>getController();
    controller.initController(this, modalStage);
    controller.setBookValues(book);

    modalStage.show();
  }

  public void showFormUserAddView() {
    URL view_form_user_add_path = AppManager.class.getResource("/views/modalAddUser.fxml");
    FXMLLoader loader = new FXMLLoader(view_form_user_add_path);
    Parent root = null;
    try {
      root = loader.load();
    }
    catch (IOException err) {
      err.printStackTrace();
    }

    Scene modalScene = new Scene(new StackPane());
    Stage modalStage = new Stage(StageStyle.TRANSPARENT);
    modalStage.initModality(Modality.APPLICATION_MODAL);
    modalScene.setRoot(root);
    modalStage.setScene(modalScene);

    FormUserAddController controller = loader.<FormUserAddController>getController();
    controller.initController(this, modalStage);

    modalStage.show();
  }

  public void showFormUserEditView(User user) {
    URL view_form_user_edit_path = AppManager.class.getResource("/views/modalEditUser.fxml");
    FXMLLoader loader = new FXMLLoader(view_form_user_edit_path);
    Parent root = null;
    try {
      root = loader.load();
    }
    catch (IOException err) {
      err.printStackTrace();
    }

    Scene modalScene = new Scene(new StackPane());
    Stage modalStage = new Stage(StageStyle.TRANSPARENT);
    modalStage.initModality(Modality.APPLICATION_MODAL);
    modalScene.setRoot(root);
    modalStage.setScene(modalScene);

    FormUserEditController controller = loader.<FormUserEditController>getController();
    controller.initController(this, modalStage);
    controller.setUserValues(user);

    modalStage.show();
  }

  public void showFormBookGiveView(Book book) {
    URL view_form_book_give_path = AppManager.class.getResource("/views/modalGiveBook.fxml");
    FXMLLoader loader = new FXMLLoader(view_form_book_give_path);
    Parent root = null;
    try {
      root = loader.load();
    }
    catch (IOException err) {
      err.printStackTrace();
    }

    Scene modalScene = new Scene(new StackPane());
    Stage modalStage = new Stage(StageStyle.TRANSPARENT);
    modalStage.initModality(Modality.APPLICATION_MODAL);
    modalScene.setRoot(root);
    modalStage.setScene(modalScene);

    FormBookGiveController controller = loader.<FormBookGiveController>getController();
    controller.initController(this, modalStage);
    controller.setBookValues(book);
    Base.open();
    List<User> users = User.findAll();
    controller.setUsersValues(users);
    Base.close();

    modalStage.show();
  }

  public void showFormBookTakeView(Book book) {
    URL view_form_book_take_path = AppManager.class.getResource("/views/modalTakeBook.fxml");
    FXMLLoader loader = new FXMLLoader(view_form_book_take_path);
    Parent root = null;
    try {
      root = loader.load();
    }
    catch (IOException err) {
      err.printStackTrace();
    }

    Scene modalScene = new Scene(new StackPane());
    Stage modalStage = new Stage(StageStyle.TRANSPARENT);
    modalStage.initModality(Modality.APPLICATION_MODAL);
    modalScene.setRoot(root);
    modalStage.setScene(modalScene);

    FormBookTakeController controller = loader.<FormBookTakeController>getController();
    controller.initController(this, modalStage);
    controller.setBooksUsersValues(book);

    modalStage.show();
  }

  public void addBookToDatabase(String name, String isbn, String author) {
    Base.open();

    Book book = new Book();
    book.set("name", name);
    book.set("isbn", isbn);
    book.set("author", author);

    book.saveIt();

    Base.close();

    this.mainController.addBookToTableView(book);
  }

  public void updateBookToDatabase(int id, String name, String isbn, String author) {
    Base.open();
    Book book = Book.findById(id);
    book.set("name", name);
    book.set("isbn", isbn);
    book.set("author", author);
    book.saveIt();
    Base.close();
    this.mainController.updateBookToTableView(book);
  }

  public void addUserToDatabase(String sname, String fname, String mname, String number) {
    Base.open();

    User user = new User();
    user.set("sname", sname);
    user.set("fname", fname);
    user.set("mname", mname);
    user.set("number", number);

    user.saveIt();

    Base.close();

    this.mainController.addUserToTableView(user);
  }

  public void updateUserToDatabase(int id, String sname, String fname, String mname, String number) {
    Base.open();
    User user = User.findById(id);
    user.set("sname", sname);
    user.set("fname", fname);
    user.set("mname", mname);
    user.set("number", number);
    user.saveIt();
    Base.close();
    this.mainController.updateUserToTableView(user);
  }

  public void addBookGiveToDatabase(int book_id, int howmanydays, User user, LocalDate taked_at) {
    Base.open();
    BooksUsers booksUsers = new BooksUsers();
    booksUsers.setInteger("book_id", book_id);
    booksUsers.setInteger("user_id", user.getInteger("id"));
    booksUsers.setDate("taked_at", taked_at);
    booksUsers.setInteger("howmanydays", howmanydays);
    booksUsers.saveIt();
    Book book = Book.findById(book_id);
    book.setBoolean("taked", true);
    book.saveIt();
    Base.close();
    this.mainController.addBooksUsersToTableView(booksUsers);
    this.mainController.updateBookToTableView(book);
  }

  public void addBookTakeToDatabase(int books_users_id, LocalDate return_at) {
    Base.open();

    BooksUsers booksUsers = BooksUsers.findById(books_users_id);
    booksUsers.setBoolean("wasreturn", true);
    booksUsers.setDate("return_at", return_at);
    booksUsers.saveIt();

    Book book = Book.findById(booksUsers.getInteger("book_id"));
    book.setBoolean("taked", false);
    book.saveIt();

    Base.close();

    this.mainController.updateBookToTableView(book);
    this.mainController.updateBooksUsersToTableView(booksUsers);
  }
}
