package org.library.app;

import org.javalite.activejdbc.Base;
import org.javalite.activejdbc.DBException;
import org.javalite.activejdbc.Model;
import org.javalite.activejdbc.annotations.Table;
import java.time.LocalDate;
import java.sql.Date;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;


@Table("books_users")
public class BooksUsers extends Model {

  public LocalDate getTakedAt() {
    return getDate("taked_at").toLocalDate();
  }

  public String getTakedAtAsStr() {
    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("dd-MM-yyyy");
    return getTakedAt().format(formatter);
  }

  public LocalDate getTakedStopAt() {
    LocalDate takedAt = getTakedAt();
    int howmanydays = getInteger("howmanydays");
    LocalDate takedStopAt = takedAt.plusDays(howmanydays);
    return takedStopAt;
  }

  public String getTakedStopAtAsStr() {
    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("dd-MM-yyyy");
    return getTakedStopAt().format(formatter);
  }

  public long getTakedTimeOut() {
    LocalDate timeNow = null;
    Date returnAt = getDate("return_at");

    if (returnAt == null) {
      timeNow = LocalDate.now();
    } else {
      timeNow = returnAt.toLocalDate();
    }

    LocalDate timeTakedStop = getTakedStopAt();
    return ChronoUnit.DAYS.between(timeNow, timeTakedStop);
  }

  public String getTakedTimeOutWithNull() {
    LocalDate timeNow = null;
    Date returnAt = getDate("return_at");

    if (returnAt == null) {
      timeNow = LocalDate.now();
    } else {
      timeNow = returnAt.toLocalDate();
    }

    LocalDate timeTakedStop = getTakedStopAt();
    long takedTimeOut = ChronoUnit.DAYS.between(timeNow, timeTakedStop);
    if (takedTimeOut < 0) {
      return "" + Math.abs(takedTimeOut);
    } else {
      return "";
    }
  }

  public String getTakedTimeOutAsStr() {
    long takedTimeOut = getTakedTimeOut();
    if (takedTimeOut > 0) {
      return "нет, ещё " + takedTimeOut + " д.";
    } else if (takedTimeOut == 0) {
      return "нет, вернуть сегодня";
    } else if (takedTimeOut < 0) {
      return "да, " + Math.abs(takedTimeOut) + " д.";
    } else {
      return "";
    }
  }

  public String getTakedTimeOutYesNo() {
    long takedTimeOut = getTakedTimeOut();
    if (takedTimeOut > 0) {
      return "нет";
    } else if (takedTimeOut == 0) {
      return "нет";
    } else if (takedTimeOut < 0) {
      return "да";
    } else {
      return "";
    }
  }

  public LocalDate getReturnAt() {
    Date returnAt = getDate("return_at");
    if (returnAt == null) {
      return null;
    } else {
      return returnAt.toLocalDate();
    }
  }

  public String getReturnAtAsStr() {
    if (getReturnAt() == null) {
      return "";
    } else {
      DateTimeFormatter formatter = DateTimeFormatter.ofPattern("dd-MM-yyyy");
      return getReturnAt().format(formatter);
    }
  }

  public Integer getHowManyDays() {
    return getInteger("howmanydays");
  }

  public String getHowManyDaysAsStr() {
    return getHowManyDays().toString();
  }

  public String getBookName() {
    Base.open();
    Book book = Book.findById(getInteger("book_id"));
    Base.close();
    return book.getString("name");
  }

  public String getBookISBN() {
    Base.open();
    Book book = Book.findById(getInteger("book_id"));
    Base.close();
    return book.getString("isbn");
  }

  public String getBookAuthor() {
    Base.open();
    Book book = Book.findById(getInteger("book_id"));
    Base.close();
    return book.getString("author");
  }

  public String getUserName() {
    Base.open();
    User user = User.findById(getInteger("user_id"));
    Base.close();
    return user.getName();
  }

  public String getUserNumber() {
    Base.open();
    User user = User.findById(getInteger("user_id"));
    Base.close();
    return user.getString("number");
  }

  public long getTotalTakedTime() {
    LocalDate timeNow = null;
    Date returnAt = getDate("return_at");

    if (returnAt == null) {
      timeNow = LocalDate.now();
    } else {
      timeNow = returnAt.toLocalDate();
    }

    LocalDate timeTakedStart = getTakedAt();
    return Math.abs( ChronoUnit.DAYS.between(timeNow, timeTakedStart) );
  }
}
