package org.library.app;

import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.Parent;
import javafx.scene.control.*;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.input.MouseEvent;
import javafx.scene.Node;
import javafx.stage.Stage;
import javafx.event.*;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

import org.javalite.activejdbc.Base;
import org.javalite.activejdbc.DBException;

import java.util.List;

import java.io.IOException;
import java.io.File;

public class MainController {
  private AppManager appManager;
  private Book currentBook;

  private ObservableList<Book> booksList = FXCollections.observableArrayList();
  private ObservableList<User> usersList = FXCollections.observableArrayList();
  private ObservableList<BooksUsers> booksUsersList = FXCollections.observableArrayList();

  @FXML private TableView<Book> tableBooksView;
  @FXML private TableColumn<Book, String> nameColumn;
  @FXML private TableColumn<Book, String> isbnColumn;
  @FXML private TableColumn<Book, String> authorColumn;
  @FXML private TableColumn<Book, String> takedAsStrColumn;
  @FXML private TableColumn<Book, String> userNameColumn;

  @FXML private TableView<User> tableUsersView;
  @FXML private TableColumn<User, String> snameColumn;
  @FXML private TableColumn<User, String> fnameColumn;
  @FXML private TableColumn<User, String> mnameColumn;
  @FXML private TableColumn<User, String> numberColumn;

  @FXML private TableView<BooksUsers> tableBooksUsersView;
  @FXML private TableColumn<BooksUsers, String> booksUsersBookNameColumn;
  @FXML private TableColumn<BooksUsers, String> booksUsersBookISBNColumn;
  @FXML private TableColumn<BooksUsers, String> booksUsersBookAuthorColumn;
  @FXML private TableColumn<BooksUsers, String> booksUsersUserNameColumn;
  @FXML private TableColumn<BooksUsers, String> booksUsersUserNumberColumn;
  @FXML private TableColumn<BooksUsers, String> booksUsersTakeStartAtColumn;
  @FXML private TableColumn<BooksUsers, String> booksUsersHowManyDaysColumn;
  @FXML private TableColumn<BooksUsers, String> booksUsersTakeStopAtColumn;
  @FXML private TableColumn<BooksUsers, String> booksUsersReturnAtColumn;
  @FXML private TableColumn<BooksUsers, String> booksUsersTimeOutColumn;
  @FXML private TableColumn<BooksUsers, String> booksUsersTimeOutDaysColumn;
  @FXML private TableColumn<BooksUsers, String> booksUsersTimeTotalColumn;

  @FXML private Label labelBookName;
  @FXML private Label labelBookIsbn;
  @FXML private Label labelBookAuthor;

  @FXML private Label labelUserBookFullname;
  @FXML private Label labelUserBookNumber;
  @FXML private Label labelUserBookTakeStartTime;
  @FXML private Label labelUserBookHowManyDays;
  @FXML private Label labelUserBookTakeStopTime;
  @FXML private Label labelUserBookTakeTimeOut;

  @FXML private Label labelUserSname;
  @FXML private Label labelUserFname;
  @FXML private Label labelUserMname;
  @FXML private Label labelUserNumber;

  @FXML private Button btnBookAdd;
  @FXML private Button btnUserAdd;
  @FXML private Button btnSignOut;

  @FXML private Button btnToEditBook;
  @FXML private Button btnToGiveBook;
  @FXML private Button btnToTakeBook;
  @FXML private Button btnToRemoveBook;

  @FXML private Button btnToEditUser;
  @FXML private Button btnToRemoveUser;

  public void initialize() {
    initBooksTable();
    initUsersTable();
    initBooksUsersTable();
  }

  private void initBooksTable() {
    nameColumn.setCellValueFactory(new PropertyValueFactory<Book, String>("name"));
    isbnColumn.setCellValueFactory(new PropertyValueFactory<Book, String>("isbn"));
    authorColumn.setCellValueFactory(new PropertyValueFactory<Book, String>("author"));
    takedAsStrColumn.setCellValueFactory(new PropertyValueFactory<Book, String>("takedAsStr"));
    userNameColumn.setCellValueFactory(new PropertyValueFactory<Book, String>("userName"));

    Base.open();
    List<Book> tmpBooksList = Book.findAll();
    for (Book item : tmpBooksList) {
      booksList.add(item);
    }
    Base.close();
    tableBooksView.setItems(booksList);
  }

  public void initUsersTable() {
    snameColumn.setCellValueFactory(new PropertyValueFactory<User, String>("sname"));
    fnameColumn.setCellValueFactory(new PropertyValueFactory<User, String>("fname"));
    mnameColumn.setCellValueFactory(new PropertyValueFactory<User, String>("mname"));
    numberColumn.setCellValueFactory(new PropertyValueFactory<User, String>("number"));

    Base.open();
    List<User> tmpUsersList = User.findAll();
    for (User item : tmpUsersList) {
      usersList.add(item);
    }
    Base.close();
    tableUsersView.setItems(usersList);
  }

  public void initBooksUsersTable() {
    booksUsersBookNameColumn.setCellValueFactory(new PropertyValueFactory<BooksUsers, String>("bookName"));
    booksUsersBookISBNColumn.setCellValueFactory(new PropertyValueFactory<BooksUsers, String>("bookISBN"));
    booksUsersBookAuthorColumn.setCellValueFactory(new PropertyValueFactory<BooksUsers, String>("bookAuthor"));
    booksUsersUserNameColumn.setCellValueFactory(new PropertyValueFactory<BooksUsers, String>("userName"));
    booksUsersUserNumberColumn.setCellValueFactory(new PropertyValueFactory<BooksUsers, String>("userNumber"));
    booksUsersTakeStartAtColumn.setCellValueFactory(new PropertyValueFactory<BooksUsers, String>("takedAtAsStr"));
    booksUsersHowManyDaysColumn.setCellValueFactory(new PropertyValueFactory<BooksUsers, String>("howManyDaysAsStr"));
    booksUsersTakeStopAtColumn.setCellValueFactory(new PropertyValueFactory<BooksUsers, String>("takedStopAtAsStr"));
    booksUsersReturnAtColumn.setCellValueFactory(new PropertyValueFactory<BooksUsers, String>("returnAtAsStr"));
    booksUsersTimeOutColumn.setCellValueFactory(new PropertyValueFactory<BooksUsers, String>("takedTimeOutYesNo"));
    booksUsersTimeOutDaysColumn.setCellValueFactory(new PropertyValueFactory<BooksUsers, String>("takedTimeOutWithNull"));
    booksUsersTimeTotalColumn.setCellValueFactory(new PropertyValueFactory<BooksUsers, String>("totalTakedTime"));

    Base.open();
    List<BooksUsers> tmpBooksUsersList = BooksUsers.findAll();
    for (BooksUsers item : tmpBooksUsersList) {
      booksUsersList.add(item);
    }
    Base.close();
    tableBooksUsersView.setItems(booksUsersList);
  }

  public void initController(final AppManager appManager) {
    this.appManager = appManager;
    final MainController self = this;

    btnBookAdd.setOnAction(new EventHandler<ActionEvent>() {
      @Override
      public void handle(ActionEvent event) {
        appManager.showFormBookAddView();
      }
    });

    btnUserAdd.setOnAction(new EventHandler<ActionEvent>() {
      @Override
      public void handle(ActionEvent event) {
        appManager.showFormUserAddView();
      }
    });

    btnSignOut.setOnAction(new EventHandler<ActionEvent>() {
      @Override
      public void handle(ActionEvent event) {
        appManager.showLoginView();
      }
    });

    btnToEditBook.setOnAction(new EventHandler<ActionEvent>() {
      @Override
      public void handle(ActionEvent event) {
        appManager.showFormBookEditView(self.currentBook);
      }
    });

    btnToGiveBook.setOnAction(new EventHandler<ActionEvent>() {
      @Override
      public void handle(ActionEvent event) {
        appManager.showFormBookGiveView(self.currentBook);
      }
    });

    btnToTakeBook.setOnAction(new EventHandler<ActionEvent>() {
      @Override
      public void handle(ActionEvent event) {
        appManager.showFormBookTakeView(self.currentBook);
      }
    });

    btnToRemoveBook.setOnAction(new EventHandler<ActionEvent>() {
      @Override
      public void handle(ActionEvent event) {
      }
    });

    tableBooksView.setOnMousePressed(new EventHandler<MouseEvent>() {
      @Override
      public void handle(MouseEvent event) {
        if (event.isPrimaryButtonDown() && event.getClickCount() == 1) {
          Node node = ((Node) event.getTarget());
          TableRow row = null;

          if (node instanceof TableRow) {
            row = (TableRow) node;
          } else if (node.getParent() instanceof TableRow) {
            row = (TableRow) node.getParent();
          } else if (node.getParent().getParent() instanceof TableRow) {
            row = (TableRow) node.getParent().getParent();
          }

          if (row == null) {
            setBooksButtonsBarActive(true, null);
            setCurrentBook(null);
          } else {
            Book item = ((Book) row.getItem());
            if (item == null) {
              setBooksButtonsBarActive(true, null);
              setCurrentBook(null);
            } else {
              setBooksButtonsBarActive(false, item);
              setCurrentBook(item);
              showBookInPane(item);
            }
          }
        }

        if (event.isPrimaryButtonDown() && event.getClickCount() == 2) {
          Node node = ((Node) event.getTarget());
          TableRow row = null;

          if (node instanceof TableRow) {
            row = (TableRow) node;
          } else if (node.getParent() instanceof TableRow) {
            row = (TableRow) node.getParent();
          } else if (node.getParent().getParent() instanceof TableRow) {
            row = (TableRow) node.getParent().getParent();
          }

          if (row == null) {
            setBooksButtonsBarActive(true, null);
            setCurrentBook(null);
          } else {
            Book item = ((Book) row.getItem());
            if (item == null) {
              setBooksButtonsBarActive(true, null);
              setCurrentBook(null);
            } else {
              setBooksButtonsBarActive(false, item);
              setCurrentBook(item);
              appManager.showFormBookEditView(item);
            }
          }
        }
      }
    });

    tableUsersView.setOnMousePressed(new EventHandler<MouseEvent>() {
      @Override
      public void handle(MouseEvent event) {
        if (event.isPrimaryButtonDown() && event.getClickCount() == 1) {
          Node node = ((Node) event.getTarget());
          TableRow row = null;

          if (node instanceof TableRow) {
            row = (TableRow) node;
          } else if (node.getParent() instanceof TableRow) {
            row = (TableRow) node.getParent();
          } else if (node.getParent().getParent() instanceof TableRow) {
            row = (TableRow) node.getParent().getParent();
          }

          if (row == null) {
          } else {
            User item = ((User) row.getItem());
            if (item == null) {
            } else {
              showUserInPane(item);
            }
          }
        }

        if (event.isPrimaryButtonDown() && event.getClickCount() == 2) {
          Node node = ((Node) event.getTarget());
          TableRow row = null;

          if (node instanceof TableRow) {
            row = (TableRow) node;
          } else if (node.getParent() instanceof TableRow) {
            row = (TableRow) node.getParent();
          } else if (node.getParent().getParent() instanceof TableRow) {
            row = (TableRow) node.getParent().getParent();
          }

          if (row == null) {
          } else {
            User item = ((User) row.getItem());
            if (item == null) {
            } else {
              appManager.showFormUserEditView(item);
            }
          }
        }
      }
    });

  }

  public void showBookInPane(Book book) {
    labelBookName.setText(book.getName());
    labelBookIsbn.setText(book.getIsbn());
    labelBookAuthor.setText(book.getAuthor());
    if (book.getBoolean("taked")) {
      User user = book.getTakedUser();
      BooksUsers booksUsers = book.getTakedRelation();

      labelUserBookFullname.setText(user.getName());
      labelUserBookNumber.setText(user.getString("number"));
      labelUserBookTakeStartTime.setText(booksUsers.getTakedAtAsStr());
      labelUserBookHowManyDays.setText(booksUsers.getString("howmanydays"));
      labelUserBookTakeStopTime.setText(booksUsers.getTakedStopAtAsStr());
      labelUserBookTakeTimeOut.setText(booksUsers.getTakedTimeOutAsStr());
    } else {
      labelUserBookFullname.setText("");
      labelUserBookNumber.setText("");
      labelUserBookTakeStartTime.setText("");
      labelUserBookHowManyDays.setText("");
      labelUserBookTakeStopTime.setText("");
      labelUserBookTakeTimeOut.setText("");
    }
  }

  public void showUserInPane(User user) {
    labelUserSname.setText(user.getSname());
    labelUserFname.setText(user.getFname());
    labelUserMname.setText(user.getMname());
    labelUserNumber.setText(user.getNumber());
  }

  public void addBookToTableView(Book book) {
    booksList.add(book);
  }

  public void updateBookToTableView(Book book) {
    int index = 0;
    int targetIndex = -1;
    for (Book item : booksList) {
      if (book.getInteger("id").equals(item.getInteger("id")) ) {
        targetIndex = index;
        break;
      }
      index += 1;
    }
    booksList.set(targetIndex, book);
    tableBooksView.getSelectionModel().clearAndSelect(targetIndex);
    tableBooksView.getFocusModel().focus(targetIndex);
    showBookInPane(book);
    setCurrentBook(book);
    setBooksButtonsBarActive(false, book);
  }

  public void updateBooksUsersToTableView(BooksUsers booksUsers) {
    int index = 0;
    int targetIndex = -1;
    for (BooksUsers item : booksUsersList) {
      if (booksUsers.getInteger("id").equals(item.getInteger("id")) ) {
        targetIndex = index;
        break;
      }
      index += 1;
    }
    if (targetIndex > -1) {
      booksUsersList.set(targetIndex, booksUsers);
      //tableBooksUsersView.getSelectionModel().clearAndSelect(targetIndex);
      //tableBooksUsersView.getFocusModel().focus(targetIndex);
    }
  }

  public void addUserToTableView(User user) {
    usersList.add(user);
  }

  public void addBooksUsersToTableView(BooksUsers booksUsers) {
    booksUsersList.add(booksUsers);
  }

  public void updateUserToTableView(User user) {
    int index = 0;
    int targetIndex = -1;
    for (User item : usersList) {
      if (user.getInteger("id").equals(item.getInteger("id")) ) {
        targetIndex = index;
        break;
      }
      index += 1;
    }
    usersList.set(targetIndex, user);
    tableUsersView.getSelectionModel().clearAndSelect(targetIndex);
    tableUsersView.getFocusModel().focus(targetIndex);
    showUserInPane(user);
  }

  public void setBooksButtonsBarActive(Boolean active, Book book) {
    btnToEditBook.setDisable(active);
    btnToRemoveBook.setDisable(true);

    if (book == null) {
      btnToGiveBook.setDisable(true);
      btnToTakeBook.setDisable(true);
    } else {
      if (book.getBoolean("taked")) {
        btnToGiveBook.setDisable(true);
        btnToTakeBook.setDisable(false);
      } else {
        btnToGiveBook.setDisable(false);
        btnToTakeBook.setDisable(true);
      }
    }
  }

  public void setCurrentBook(Book book) {
    this.currentBook = book;
  }

}
